import os
from sudoku import SudokuBoard

class SolverRunner:
    
    def __init__(self, solver_class, tasks_dir, solutions_dir):
        self.solver_class = solver_class
        self.tasks_dirpath = os.path.abspath(tasks_dir)
        self.solutions_dirpath = os.path.abspath(solutions_dir)
        
    def solve_single_task(self, task_filepath):
        # Prepare the solution name and directory structure
        solution_filepath = task_filepath.replace(
            self.tasks_dirpath, self.solutions_dirpath)
        os.makedirs(os.path.dirname(solution_filepath), exist_ok=True)
        # Load the sudoku task
        board = SudokuBoard()
        board.load(task_filepath)
        # Create solver and solve the task
        solver = self.solver_class()
        solved_board = solver.solve(board)
        # Write the solution
        solved_board.save(solution_filepath)
        # Print info
        print('The solution of', task_filepath, 'written to', solution_filepath+'.')
        
    def run(self):
        for tasks_subdirpath, dirnames, filenames in os.walk(self.tasks_dirpath):
            # Walk all the tasks in the current directory and apply the given function on them.
            for fname in filenames:
                if not fname.endswith('.ss'): continue
                task_filepath = os.path.join(tasks_subdirpath, fname)
                self.solve_single_task(task_filepath)
        